@@ -101,11 +101,15 @@ static snd_pcm_uframes_t buffer_frames = 0;
 static int avail_min = -1;
 static int start_delay = 0;
 static int stop_delay = 0;
+static int monotonic = 0;
 static int verbose = 0;
 static int vumeter = VUMETER_NONE;
 static int buffer_pos = 0;
 static size_t bits_per_sample, bits_per_frame;
 static size_t chunk_bytes;
+static int test_position = 0;
+static int test_coef = 8;
+static int test_nowait = 0;
 static snd_output_t *log;
 
 static int fd = -1;
@@ -126,7 +130,7 @@ static void end_wave(int fd);
 static void begin_au(int fd, size_t count);
 static void end_au(int fd);
 
-struct fmt_capture {
+static const struct fmt_capture {
 	void (*start) (int fd, size_t count);
 	void (*end) (int fd);
 	char *what;
@@ -186,7 +190,11 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 "    --disable-resample  disable automatic rate resample\n"
 "    --disable-channels  disable automatic channel conversions\n"
 "    --disable-format    disable automatic format conversions\n"
-"    --disable-softvol   disable software volume control (softvol)\n")
+"    --disable-softvol   disable software volume control (softvol)\n"
+"    --test-position     test ring buffer position\n"
+"    --test-coef=#	 test coeficient for ring buffer position (default 8)\n"
+"                        expression for validation is: coef * (buffer_size / 2)\n"
+"    --test-nowait       do not wait for ring buffer - eats whole CPU\n")
 		, command);
 	printf(_("Recognized sample formats are:"));
 	for (k = 0; k < SND_PCM_FORMAT_LAST; ++k) {
@@ -285,7 +293,7 @@ static void pcm_list(void)
 		name = snd_device_name_get_hint(*n, "NAME");
 		descr = snd_device_name_get_hint(*n, "DESC");
 		io = snd_device_name_get_hint(*n, "IOID");
-		if (io != NULL && strcmp(io, filter) == 0)
+		if (io != NULL && strcmp(io, filter) != 0)
 			goto __end;
 		printf("%s\n", name);
 		if ((descr1 = descr) != NULL) {
@@ -347,14 +355,17 @@ enum {
 	OPT_DISABLE_RESAMPLE,
 	OPT_DISABLE_CHANNELS,
 	OPT_DISABLE_FORMAT,
-	OPT_DISABLE_SOFTVOL
+	OPT_DISABLE_SOFTVOL,
+	OPT_TEST_POSITION,
+	OPT_TEST_COEF,
+	OPT_TEST_NOWAIT
 };
 
 int main(int argc, char *argv[])
 {
 	int option_index;
-	char *short_options = "hnlLD:qt:c:f:r:d:MNF:A:R:T:B:vV:IPC";
-	static struct option long_options[] = {
+	static const char short_options[] = "hnlLD:qt:c:f:r:d:MNF:A:R:T:B:vV:IPC";
+	static const struct option long_options[] = {
 		{"help", 0, 0, 'h'},
 		{"version", 0, 0, OPT_VERSION},
 		{"list-devnames", 0, 0, 'n'},
@@ -385,6 +396,9 @@ int main(int argc, char *argv[])
 		{"disable-channels", 0, 0, OPT_DISABLE_CHANNELS},
 		{"disable-format", 0, 0, OPT_DISABLE_FORMAT},
 		{"disable-softvol", 0, 0, OPT_DISABLE_SOFTVOL},
+		{"test-position", 0, 0, OPT_TEST_POSITION},
+		{"test-coef", 1, 0, OPT_TEST_COEF},
+		{"test-nowait", 0, 0, OPT_TEST_NOWAIT},
 		{0, 0, 0, 0}
 	};
 	char *pcm_name = "default";
@@ -563,6 +577,17 @@ int main(int argc, char *argv[])
 		case OPT_DISABLE_SOFTVOL:
 			open_mode |= SND_PCM_NO_SOFTVOL;
 			break;
+		case OPT_TEST_POSITION:
+			test_position = 1;
+			break;
+		case OPT_TEST_COEF:
+			test_coef = strtol(optarg, NULL, 0);
+			if (test_coef < 1)
+				test_coef = 1;
+			break;
+		case OPT_TEST_NOWAIT:
+			test_nowait = 1;
+			break;
 		default:
 			fprintf(stderr, _("Try `%s --help' for more information.\n"), command);
 			return 1;
@@ -656,7 +681,7 @@ int main(int argc, char *argv[])
  * Safe read (for pipes)
  */
  
-ssize_t safe_read(int fd, void *buf, size_t count)
+static ssize_t safe_read(int fd, void *buf, size_t count)
 {
 	ssize_t result = 0, res;
 
@@ -694,7 +719,7 @@ static int test_vocfile(void *buffer)
  * helper for test_wavefile
  */
 
-size_t test_wavefile_read(int fd, u_char *buffer, size_t *size, size_t reqsize, int line)
+static size_t test_wavefile_read(int fd, u_char *buffer, size_t *size, size_t reqsize, int line)
 {
 	if (*size >= reqsize)
 		return *size;
@@ -1010,6 +1035,7 @@ static void set_params(void)
 							     &buffer_frames);
 	}
 	assert(err >= 0);
+	monotonic = snd_pcm_hw_params_is_monotonic(params);
 	err = snd_pcm_hw_params(handle, params);
 	if (err < 0) {
 		error(_("Unable to install hw params:"));
@@ -1073,6 +1099,24 @@ static void set_params(void)
 		if (hwparams.channels != 2 || !interleaved || verbose > 2)
 			vumeter = VUMETER_MONO;
 	}
+
+	/* show mmap buffer arragment */
+	if (mmap_flag && verbose) {
+		const snd_pcm_channel_area_t *areas;
+		snd_pcm_uframes_t offset;
+		int i;
+		err = snd_pcm_mmap_begin(handle, &areas, &offset, &chunk_size);
+		if (err < 0) {
+			error("snd_pcm_mmap_begin problem: %s", snd_strerror(err));
+			exit(EXIT_FAILURE);
+		}
+		for (i = 0; i < hwparams.channels; i++)
+			fprintf(stderr, "mmap_area[%i] = %p,%u,%u (%u)\n", i, areas[i].addr, areas[i].first, areas[i].step, snd_pcm_format_physical_width(hwparams.format));
+		/* not required, but for sure */
+		snd_pcm_mmap_commit(handle, offset, 0);
+	}
+
+	buffer_frames = buffer_size;	/* for position test */
 }
 
 #ifndef timersub
@@ -1087,6 +1131,18 @@ do { \
 } while (0)
 #endif
 
+#ifndef timermsub
+#define	timermsub(a, b, result) \
+do { \
+	(result)->tv_sec = (a)->tv_sec - (b)->tv_sec; \
+	(result)->tv_nsec = (a)->tv_nsec - (b)->tv_nsec; \
+	if ((result)->tv_nsec < 0) { \
+		--(result)->tv_sec; \
+		(result)->tv_nsec += 1000000000L; \
+	} \
+} while (0)
+#endif
+
 /* I/O error handler */
 static void xrun(void)
 {
@@ -1099,13 +1155,27 @@ static void xrun(void)
 		exit(EXIT_FAILURE);
 	}
 	if (snd_pcm_status_get_state(status) == SND_PCM_STATE_XRUN) {
-		struct timeval now, diff, tstamp;
-		gettimeofday(&now, 0);
-		snd_pcm_status_get_trigger_tstamp(status, &tstamp);
-		timersub(&now, &tstamp, &diff);
-		fprintf(stderr, _("%s!!! (at least %.3f ms long)\n"),
-			stream == SND_PCM_STREAM_PLAYBACK ? _("underrun") : _("overrun"),
-			diff.tv_sec * 1000 + diff.tv_usec / 1000.0);
+		if (monotonic) {
+#ifdef HAVE_CLOCK_GETTIME
+			struct timespec now, diff, tstamp;
+			clock_gettime(CLOCK_MONOTONIC, &now);
+			snd_pcm_status_get_trigger_htstamp(status, &tstamp);
+			timermsub(&now, &tstamp, &diff);
+			fprintf(stderr, _("%s!!! (at least %.3f ms long)\n"),
+				stream == SND_PCM_STREAM_PLAYBACK ? _("underrun") : _("overrun"),
+				diff.tv_sec * 1000 + diff.tv_nsec / 10000000.0);
+#else
+			fprintf(stderr, "%s !!!\n", _("underrun"));
+#endif
+		} else {
+			struct timeval now, diff, tstamp;
+			gettimeofday(&now, 0);
+			snd_pcm_status_get_trigger_tstamp(status, &tstamp);
+			timersub(&now, &tstamp, &diff);
+			fprintf(stderr, _("%s!!! (at least %.3f ms long)\n"),
+				stream == SND_PCM_STREAM_PLAYBACK ? _("underrun") : _("overrun"),
+				diff.tv_sec * 1000 + diff.tv_usec / 1000.0);
+		}
 		if (verbose) {
 			fprintf(stderr, _("Status:\n"));
 			snd_pcm_status_dump(status, log);
@@ -1369,6 +1439,65 @@ static void compute_max_peak(u_char *data, size_t count)
 	}
 }
 
+static void do_test_position(void)
+{
+	static long counter = 0;
+	static time_t tmr = -1;
+	time_t now;
+	static float availsum, delaysum, samples;
+	static snd_pcm_sframes_t maxavail, maxdelay;
+	static snd_pcm_sframes_t minavail, mindelay;
+	static snd_pcm_sframes_t badavail = 0, baddelay = 0;
+	snd_pcm_sframes_t outofrange;
+	snd_pcm_sframes_t avail, delay;
+	int err;
+
+	err = snd_pcm_avail_delay(handle, &avail, &delay);
+	if (err < 0)
+		return;
+	outofrange = (test_coef * (snd_pcm_sframes_t)buffer_frames) / 2;
+	if (avail > outofrange || avail < -outofrange ||
+	    delay > outofrange || delay < -outofrange) {
+	  badavail = avail; baddelay = delay;
+	  availsum = delaysum = samples = 0;
+	  maxavail = maxdelay = 0;
+	  minavail = mindelay = buffer_frames * 16;
+	  fprintf(stderr, _("Suspicious buffer position (%li total): "
+	  	"avail = %li, delay = %li, buffer = %li\n"),
+	  	++counter, (long)avail, (long)delay, (long)buffer_frames);
+	} else if (verbose) {
+		time(&now);
+		if (tmr == (time_t) -1) {
+			tmr = now;
+			availsum = delaysum = samples = 0;
+			maxavail = maxdelay = 0;
+			minavail = mindelay = buffer_frames * 16;
+		}
+		if (avail > maxavail)
+			maxavail = avail;
+		if (delay > maxdelay)
+			maxdelay = delay;
+		if (avail < minavail)
+			minavail = avail;
+		if (delay < mindelay)
+			mindelay = delay;
+		availsum += avail;
+		delaysum += delay;
+		samples++;
+		if (avail != 0 && now != tmr) {
+			fprintf(stderr, "BUFPOS: avg%li/%li "
+				"min%li/%li max%li/%li (%li) (%li:%li/%li)\n",
+				(long)(availsum / samples),
+				(long)(delaysum / samples),
+				(long)minavail, (long)mindelay,
+				(long)maxavail, (long)maxdelay,
+				(long)buffer_frames,
+				counter, badavail, baddelay);
+			tmr = now;
+		}
+	}
+}
+
 /*
  *  write function
  */
@@ -1383,9 +1512,14 @@ static ssize_t pcm_write(u_char *data, size_t count)
 		count = chunk_size;
 	}
 	while (count > 0) {
+		if (test_position)
+			do_test_position();
 		r = writei_func(handle, data, count);
+		if (test_position)
+			do_test_position();
 		if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
-			snd_pcm_wait(handle, 1000);
+			if (!test_nowait)
+				snd_pcm_wait(handle, 1000);
 		} else if (r == -EPIPE) {
 			xrun();
 		} else if (r == -ESTRPIPE) {
@@ -1424,9 +1558,14 @@ static ssize_t pcm_writev(u_char **data, unsigned int channels, size_t count)
 		size_t offset = result;
 		for (channel = 0; channel < channels; channel++)
 			bufs[channel] = data[channel] + offset * bits_per_sample / 8;
+		if (test_position)
+			do_test_position();
 		r = writen_func(handle, bufs, count);
+		if (test_position)
+			do_test_position();
 		if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
-			snd_pcm_wait(handle, 1000);
+			if (!test_nowait)
+				snd_pcm_wait(handle, 1000);
 		} else if (r == -EPIPE) {
 			xrun();
 		} else if (r == -ESTRPIPE) {
@@ -1462,9 +1601,14 @@ static ssize_t pcm_read(u_char *data, size_t rcount)
 	}
 
 	while (count > 0) {
+		if (test_position)
+			do_test_position();
 		r = readi_func(handle, data, count);
+		if (test_position)
+			do_test_position();
 		if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
-			snd_pcm_wait(handle, 1000);
+			if (!test_nowait)
+				snd_pcm_wait(handle, 1000);
 		} else if (r == -EPIPE) {
 			xrun();
 		} else if (r == -ESTRPIPE) {
@@ -1500,9 +1644,14 @@ static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
 		size_t offset = result;
 		for (channel = 0; channel < channels; channel++)
 			bufs[channel] = data[channel] + offset * bits_per_sample / 8;
+		if (test_position)
+			do_test_position();
 		r = readn_func(handle, bufs, count);
+		if (test_position)
+			do_test_position();
 		if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
-			snd_pcm_wait(handle, 1000);
+			if (!test_nowait)
+				snd_pcm_wait(handle, 1000);
 		} else if (r == -EPIPE) {
 			xrun();
 		} else if (r == -ESTRPIPE) {
@@ -2069,7 +2218,7 @@ static void header(int rtype, char *name)
 
 /* playing raw data */
 
-void playback_go(int fd, size_t loaded, off64_t count, int rtype, char *name)
+static void playback_go(int fd, size_t loaded, off64_t count, int rtype, char *name)
 {
 	int l, r;
 	off64_t written = 0;
@@ -2309,7 +2458,7 @@ static void capture(char *orig_name)
 	} while ((file_type == FORMAT_RAW && !timelimit) || count > 0);
 }
 
-void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count, int rtype, char **names)
+static void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count, int rtype, char **names)
 {
 	int r;
 	size_t vsize;
@@ -2361,7 +2510,7 @@ void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count,
 	snd_pcm_nonblock(handle, nonblock);
 }
 
-void capturev_go(int* fds, unsigned int channels, off64_t count, int rtype, char **names)
+static void capturev_go(int* fds, unsigned int channels, off64_t count, int rtype, char **names)
 {
 	size_t c;
 	ssize_t r;
